@@ -238,9 +238,7 @@ void KQuickStyleItem::initStyleOption()
     }
 
     case Splitter: {
-        if (!m_styleoption) {
-            m_styleoption = new QStyleOption;
-        }
+        // Use defaults
         break;
     }
 
@@ -1220,6 +1218,11 @@ QSize KQuickStyleItem::sizeFromContents(int width, int height)
     case Item: // fall through
         size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_ItemViewItem, m_styleoption, QSize(width, height));
         break;
+    case Splitter: {
+        const auto hw = handleWidth();
+        size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_Splitter, m_styleoption, QSize(hw, hw));
+        break;
+    }
     case MenuBarItem:
         size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_MenuBarItem, m_styleoption, QSize(width, height));
         break;
@@ -1243,34 +1246,54 @@ QSize KQuickStyleItem::sizeFromContents(int width, int height)
     return size.expandedTo(QSize(m_contentWidth, m_contentHeight));
 }
 
-qreal KQuickStyleItem::baselineOffset()
+int KQuickStyleItem::handleWidth() const
+{
+    // TODO: emulate QSplitter::handleWidth using contentWidth or a custom property?
+    return style()->pixelMetric(QStyle::PM_SplitterWidth, nullptr);
+}
+
+qreal KQuickStyleItem::baselineOffset() const
 {
     QRect r;
     bool ceilResult = true; // By default baseline offset rounding is done upwards
     switch (m_itemType) {
-    case RadioButton:
-        r = KQuickStyleItem::style()->subElementRect(QStyle::SE_RadioButtonContents, m_styleoption);
-        break;
     case Button:
         r = KQuickStyleItem::style()->subElementRect(QStyle::SE_PushButtonContents, m_styleoption);
         break;
     case CheckBox:
         r = KQuickStyleItem::style()->subElementRect(QStyle::SE_CheckBoxContents, m_styleoption);
         break;
-    case Edit:
-        r = KQuickStyleItem::style()->subElementRect(QStyle::SE_LineEditContents, m_styleoption);
-        break;
     case ComboBox:
-        if (const QStyleOptionComboBox *combo = qstyleoption_cast<const QStyleOptionComboBox *>(m_styleoption)) {
-            r = KQuickStyleItem::style()->subControlRect(QStyle::CC_ComboBox, combo, QStyle::SC_ComboBoxEditField);
+        if (const auto option = qstyleoption_cast<const QStyleOptionComboBox *>(m_styleoption)) {
+            r = KQuickStyleItem::style()->subControlRect(QStyle::CC_ComboBox, option, QStyle::SC_ComboBoxEditField);
             if (styleName() != QLatin1String("mac")) {
                 r.adjust(0, 0, 0, 1);
             }
         }
         break;
+    case DelayButton:
+    case ToolButton:
+        if (const auto option = qstyleoption_cast<const QStyleOptionComplex *>(m_styleoption)) {
+            r = KQuickStyleItem::style()->subControlRect(QStyle::CC_ToolButton, option, QStyle::SC_ToolButton);
+        }
+        break;
+    case Edit:
+        r = KQuickStyleItem::style()->subElementRect(QStyle::SE_LineEditContents, m_styleoption);
+        break;
+    case ProgressBar:
+        r = m_styleoption->rect;
+        r.adjust(0, 0, 0, -2);
+        break;
+    case RadioButton:
+        r = KQuickStyleItem::style()->subElementRect(QStyle::SE_RadioButtonContents, m_styleoption);
+        break;
+    case Slider:
+        r = m_styleoption->rect;
+        r.adjust(0, 0, 0, -2);
+        break;
     case SpinBox:
-        if (const QStyleOptionSpinBox *spinbox = qstyleoption_cast<const QStyleOptionSpinBox *>(m_styleoption)) {
-            r = KQuickStyleItem::style()->subControlRect(QStyle::CC_SpinBox, spinbox, QStyle::SC_SpinBoxEditField);
+        if (const auto option = qstyleoption_cast<const QStyleOptionSpinBox *>(m_styleoption)) {
+            r = KQuickStyleItem::style()->subControlRect(QStyle::CC_SpinBox, option, QStyle::SC_SpinBoxEditField);
             ceilResult = false;
         }
         break;
@@ -1730,11 +1753,7 @@ void KQuickStyleItem::paint(QPainter *painter)
         KQuickStyleItem::style()->drawPrimitive(QStyle::PE_PanelScrollAreaCorner, m_styleoption, painter);
         break;
     case Splitter:
-        if (m_styleoption->rect.width() == 1) {
-            painter->fillRect(0, 0, width(), height(), m_styleoption->palette.dark().color());
-        } else {
-            KQuickStyleItem::style()->drawControl(QStyle::CE_Splitter, m_styleoption, painter);
-        }
+        KQuickStyleItem::style()->drawControl(QStyle::CE_Splitter, m_styleoption, painter);
         break;
     case ComboBox: {
         KQuickStyleItem::style()->drawComplexControl(QStyle::CC_ComboBox, qstyleoption_cast<QStyleOptionComplex *>(m_styleoption), painter);
@@ -2001,8 +2020,10 @@ QSGNode *KQuickStyleItem::updatePaintNode(QSGNode *node, UpdatePaintNodeData *)
 void KQuickStyleItem::updatePolish()
 {
     if (isVisible() && width() >= 1 && height() >= 1) { // Note these are reals so 1 pixel is minimum
+#if QT_VERSION < QT_VERSION_CHECK(6, 7, 3)
         // Need to request the focus reason every time, the signal is not entirely reliable: https://bugreports.qt.io/browse/QTBUG-125725
         updateFocusReason();
+#endif
         const qreal devicePixelRatio = window() ? window()->effectiveDevicePixelRatio() : qApp->devicePixelRatio();
         const QSize size = QSize(m_textureWidth > 0 ? m_textureWidth : width(), m_textureHeight > 0 ? m_textureHeight : height()) * devicePixelRatio;
 
